/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsIPrefBranch2.idl"

/**
 * An empty interface to provide backwards compatibility for existing code that
 * bsmedberg didn't want to break all at once. Don't use me!
 *
 * @status NON-FROZEN interface WHICH WILL PROBABLY GO AWAY.
 */

[scriptable, uuid(355bd1e9-248a-438b-809d-e0db1b287882)]
interface nsIPrefBranchInternal : nsIPrefBranch2
{
};
